import subprocess
import platform 
import os,sys

def get_ndcg_score(txt):
  for ln in txt.split('\n'):
    ln = ln.strip()
    fields = ln.split('\t')
    metric = fields[0].strip()
    if metric == 'ndcg': 
      return ln

def run_trec(this_os,qrel_file,run_file,verbose):
  if this_os == 'Linux':
    tbin = './trec_eval.linux'
  elif this_os == 'Windows':
    tbin = 'trec_eval.exe'
  elif this_os == 'Darwin':
    tbin = './trec_eval.osx'
  else:
    print('OS is not known')

  try:
    args = (tbin, "-m", "all_trec", 
            qrel_file, run_file)
    popen = subprocess.Popen(args, stdout=subprocess.PIPE)
    popen.wait()
    output = popen.stdout.read()
    txt = output.decode()
    if verbose == True:
      print (txt)
    else:
      print (get_ndcg_score(txt))
  except Exception as e:
    print('[ERROR]: subprocess failed')
    print('[ERROR]: {}'.format(e))

def read_sort_run(run_file):
  qdic = {} 
  lines = []
  with open(run_file,'r') as f:
    for ln in f:
      ln = ln.strip()
      #print (ln)
      x,y,z = ln.split('\t')
      if x in qdic:
        qdic[x].append((y,float(z)))
      else:
        qdic[x] = []
        qdic[x].append((y,float(z)))

  rank = 1 
  for k,v in qdic.items():
    v.sort(key=lambda x:x[1], reverse=True)
    rank = 1
    for a,b in v:
        out = str(k) + ' Q0 ' + a + ' ' + str(rank) + ' ' + str(b) + ' e76767'
        lines.append(out) 
        rank += 1
  return '\n'.join(lines)

if __name__ == "__main__":
  verbose = False 
  this_os = platform.system()
  qrel_file = 'train.qrels'
  output_file = None
  if len(sys.argv) < 2:
    print('[USAGE]: python evaluate.py <vali.tsv>')
  argc = len(sys.argv) - 1
  while argc > 0:
    item = sys.argv[argc]
    #print (item)
    if item.lower() == 'verbose':
      verbose = True
    else:
      run_file = item
    argc = argc - 1
  if verbose == True:
    print ('OS = ',this_os)
    print ('qrel file = ',qrel_file)
    print ('system run file = ',run_file)
  qrel_file = 'train.qrels'
  rf = read_sort_run(run_file)
  if output_file == None:
    output_file = 'current.run'
    with open(output_file,'w') as f:
        f.write(rf)
  run_trec(this_os,qrel_file,output_file,verbose)


